# frozen_string_literal: true

require 'cms_scanner/target/server/generic'
require 'cms_scanner/target/server/apache'
require 'cms_scanner/target/server/iis'
require 'cms_scanner/target/server/nginx'
