/**
 * @file version.h
 *
 * The version of the Prism library.
 */
#ifndef PRISM_VERSION_H
#define PRISM_VERSION_H

/**
 * The major version of the Prism library as an int.
 */
#define PRISM_VERSION_MAJOR 1

/**
 * The minor version of the Prism library as an int.
 */
#define PRISM_VERSION_MINOR 7

/**
 * The patch version of the Prism library as an int.
 */
#define PRISM_VERSION_PATCH 0

/**
 * The version of the Prism library as a constant string.
 */
#define PRISM_VERSION "1.7.0"

#endif
