"banner": {
"description": "WordPress Security Scanner by the WPScan Team",
  "version": <%= WPScan::VERSION.to_json %>,
  "authors": [
    "@_WPScan_",
    "@ethicalhack3r",
    "@erwan_lr",
    "@firefart"
  ],
  "sponsor": <%= WPScan::DB::Sponsor.text.to_json %>
},
