# frozen_string_literal: true

require "active_support/core_ext/time/acts_like"
require "active_support/core_ext/time/calculations"
require "active_support/core_ext/time/compatibility"
require "active_support/core_ext/time/conversions"
require "active_support/core_ext/time/zones"
